"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const difficultys = [
    { TEXT: 'BASIC', COLOR: '#19aa19', BACKGROUND: '#e8f6e8' },
    { TEXT: 'ADVANCED', COLOR: '#f55000', BACKGROUND: '#feede5' },
    { TEXT: 'EXPERT', COLOR: '#a00a50', BACKGROUND: '#F5E6ED' },
    { TEXT: 'MASTER', COLOR: '#8200dc', BACKGROUND: '#f2e5fb' },
    { TEXT: "WORLD'S END", COLOR: '#000000', BACKGROUND: '#e5e5e5' }
];
const SupportMeta = [
    'SONGID',
    'TITLE',
    'ARTIST',
    'DESIGNER',
    'DIFFICULTY',
    'PLAYLEVEL',
    'WAVE',
    'WAVEOFFSET',
    'JACKET',
    'BACKGROUND',
    'MOVIE',
    'MOVIEOFFSET',
    'BASEBPM'
];
const RequiredMeta = [
    'SONGID',
    'TITLE',
    'ARTIST',
    'DESIGNER',
    'DIFFICULTY',
    'PLAYLEVEL'
];
const MoS = [
    // Meta of String
    'ARTIST',
    'BACKGROUND',
    'DESIGNER',
    'JACKET',
    'MOVIE',
    'SONGID',
    'TITLE',
    'WAVE'
];
const MoN = [
    // Meta of Number
    'BASEBPM',
    'MOVIEOFFSET',
    'WAVEOFFSET'
];
/**
 * メタデータのオブジェクトを返す関数
 * @param {String} sus - sus
 * @return {ISusMeta} メタデータ
 */
function getMeta(sus) {
    return getValidMetaLines(sus).reduce((obj, line) => {
        if (MoS.indexOf(line.key) > -1) {
            obj[line.key] = line.value;
        }
        if (MoN.indexOf(line.key) > -1 && Number(line.value)) {
            obj[line.key] = Number(line.value);
        }
        if (line.key === 'DIFFICULTY') {
            const difLevel = Number(line.value.slice(0, 1));
            if (isNaN(difLevel) || 0 > Number(difLevel) || Number(difLevel) > 4) {
                return obj;
            }
            obj.DIFFICULTY = Object.assign({}, difficultys[difLevel], { LEVEL: Number(difLevel), MARK: Number(difLevel) === 4
                    ? line.value.length !== 1
                        ? `${line.value}:`.split(':')[1]
                        : ''
                    : undefined });
        }
        if (line.key === 'PLAYLEVEL') {
            if (!Number(line.value) && !Number(line.value.slice(0, -1))) {
                return obj;
            }
            obj.PLAYLEVEL = {
                LEVEL: Number(line.value) || Number(line.value.slice(0, -1)),
                PLUS: /\+$/.test(line.value),
                TEXT: line.value
            };
        }
        return obj;
    }, {});
}
exports.getMeta = getMeta;
/**
 * sus譜面が有効かどうかを返す関数
 * @param {String} sus - sus
 * @return {ISusValidity} 結果
 */
function validate(sus) {
    const meta = getMeta(sus);
    const required = RequiredMeta.slice();
    if (meta.WAVE != null) {
        required.push('WAVEOFFSET');
    }
    if (meta.MOVIE != null) {
        required.push('MOVIEOFFSET');
    }
    if (meta.DIFFICULTY != null && meta.DIFFICULTY.LEVEL === 4) {
        required.splice(required.indexOf('PLAYLEVEL'), 1);
    }
    const missingMeta = required.filter(line => !meta.hasOwnProperty(line));
    const validity = {
        MISSING_META: missingMeta,
        VALIDITY: missingMeta.length === 0
    };
    return validity;
}
exports.validate = validate;
/**
 * sus文字列から有効行を配列で返す関数
 * @param {String} sus - sus
 * @return {IValidSusLine[]} sus有効行の配列
 */
function getValidMetaLines(sus) {
    return sus
        .split('\n')
        .filter(line => line.slice(0, 1) === '#') // sus有効行
        .map(line => line.slice(1)) // #除去
        .filter(line => !line.match(/^\d{3}/)) // 上3桁が数字(譜面データ)除外
        .reduce((list, line) => {
        const key = line
            .slice(0, line.indexOf(' '))
            .trim()
            .toUpperCase();
        if (isSusMeta(key)) {
            const a = {
                key,
                value: line.slice(line.indexOf(' ') + 1, line.length).trim()
            };
            list.push(a);
        }
        return list;
    }, [])
        .map(line => {
        line.value = line.value.match(/^".*"$/)
            ? line.value.slice(1, line.value.length - 1)
            : line.value;
        return line;
    }) // 文字列リテラルのダブルコーテーション外す
        .filter(line => line.value);
}
function isSusMeta(s) {
    return SupportMeta.indexOf(s) > -1;
}
//# sourceMappingURL=meta.js.map